@@ -21,8 +21,8 @@ import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.fail;
 
 import java.nio.charset.Charset;
+import java.nio.charset.StandardCharsets;
 
-import org.apache.commons.codec.Charsets;
 import org.apache.commons.codec.DecoderException;
 import org.apache.commons.codec.EncoderException;
 import org.junit.After;
@@ -32,37 +32,36 @@ import org.junit.Test;
 /**
  * TestCase for BinaryCodec class.
  *
- * @version $Id: BinaryCodecTest.java 1744166 2016-05-16 22:59:35Z sebb $
  */
 public class BinaryCodecTest {
 
-    private static final Charset CHARSET_UTF8 = Charsets.UTF_8;
+    private static final Charset CHARSET_UTF8 = StandardCharsets.UTF_8;
 
-    /** mask with bit zero based index 0 raised */
+    /** Mask with bit zero-based index 0 raised. */
     private static final int BIT_0 = 0x01;
 
-    /** mask with bit zero based index 0 raised */
+    /** Mask with bit zero-based index 1 raised. */
     private static final int BIT_1 = 0x02;
 
-    /** mask with bit zero based index 0 raised */
+    /** Mask with bit zero-based index 2 raised. */
     private static final int BIT_2 = 0x04;
 
-    /** mask with bit zero based index 0 raised */
+    /** Mask with bit zero-based index 3 raised. */
     private static final int BIT_3 = 0x08;
 
-    /** mask with bit zero based index 0 raised */
+    /** Mask with bit zero-based index 4 raised. */
     private static final int BIT_4 = 0x10;
 
-    /** mask with bit zero based index 0 raised */
+    /** Mask with bit zero-based index 5 raised. */
     private static final int BIT_5 = 0x20;
 
-    /** mask with bit zero based index 0 raised */
+    /** Mask with bit zero-based index 6 raised. */
     private static final int BIT_6 = 0x40;
 
-    /** mask with bit zero based index 0 raised */
+    /** Mask with bit zero-based index 7 raised. */
     private static final int BIT_7 = 0x80;
 
-    /** an instance of the binary codec */
+    /** An instance of the binary codec. */
     BinaryCodec instance = null;
 
     @Before
