@@ -18,14 +18,24 @@
 package org.apache.commons.codec.digest;
 
 import static org.apache.commons.codec.binary.StringUtils.getBytesUtf8;
+import static org.junit.Assert.assertArrayEquals;
 import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertTrue;
+import static org.junit.Assert.assertFalse;
+import static org.junit.Assert.assertNotNull;
 
 import java.io.ByteArrayInputStream;
 import java.io.File;
+import java.io.FileInputStream;
 import java.io.FileOutputStream;
 import java.io.IOException;
+import java.io.RandomAccessFile;
 import java.nio.ByteBuffer;
+import java.nio.file.Files;
+import java.nio.file.Path;
+import java.nio.file.Paths;
 import java.security.MessageDigest;
+import java.util.Locale;
 import java.util.Random;
 
 import org.apache.commons.codec.binary.Hex;
@@ -40,18 +50,27 @@ import org.junit.Test;
 /**
  * Tests DigestUtils methods.
  *
- * @version $Id: DigestUtilsTest.java 1811343 2017-10-06 15:19:35Z ggregory $
  */
 public class DigestUtilsTest {
 
+    private static final String EMPTY_STRING = "";
+
     private final byte[] testData = new byte[1024 * 1024];
 
     private File testFile;
 
+    private File testRandomAccessFile;
+
+    private RandomAccessFile testRandomAccessFileWrapper;
+
     private void assumeJava8() {
         Assume.assumeTrue(SystemUtils.isJavaVersionAtLeast(JavaVersion.JAVA_1_8));
     }
 
+    private void assumeJava9() {
+        Assume.assumeTrue(SystemUtils.isJavaVersionAtLeast(JavaVersion.JAVA_9));
+    }
+
     byte[] getTestData() {
         return testData;
     }
@@ -60,13 +79,27 @@ public class DigestUtilsTest {
         return testFile;
     }
 
+    Path getTestPath() {
+        return testFile.toPath();
+    }
+
+    RandomAccessFile getTestRandomAccessFile() {
+        return testRandomAccessFileWrapper;
+    }
+
     @Before
     public void setUp() throws Exception {
         new Random().nextBytes(testData);
         testFile = File.createTempFile(DigestUtilsTest.class.getName(), ".dat");
-        final FileOutputStream fos = new FileOutputStream(testFile);
-        fos.write(testData);
-        fos.close();
+        try (final FileOutputStream fos = new FileOutputStream(testFile)) {
+            fos.write(testData);
+        }
+
+        testRandomAccessFile = File.createTempFile(DigestUtilsTest.class.getName(), ".dat");
+        try (final FileOutputStream fos = new FileOutputStream(testRandomAccessFile)) {
+            fos.write(testData);
+        }
+        testRandomAccessFileWrapper = new RandomAccessFile(testRandomAccessFile, "rw");
     }
 
     @After
@@ -74,6 +107,31 @@ public class DigestUtilsTest {
         if (!testFile.delete()) {
             testFile.deleteOnExit();
         }
+
+        if (!testRandomAccessFile.delete()) {
+            testRandomAccessFile.deleteOnExit();
+        }
+    }
+
+    @Test
+    public void testDigestAs() throws IOException {
+        final String expected = "d41d8cd98f00b204e9800998ecf8427e";
+        final String pathname = "src/test/resources/empty.bin";
+        final String algo = MessageDigestAlgorithms.MD5;
+        assertEquals(expected, new DigestUtils(algo).digestAsHex(new File(pathname)));
+        try (final FileInputStream inputStream = new FileInputStream(pathname)) {
+            assertEquals(expected, new DigestUtils(algo).digestAsHex(inputStream));
+        }
+        final byte[] allBytes = Files.readAllBytes(Paths.get(pathname));
+        assertEquals(expected, new DigestUtils(algo).digestAsHex(allBytes));
+        assertEquals(expected, new DigestUtils(algo).digestAsHex(ByteBuffer.wrap(allBytes)));
+    }
+
+    @Test
+    public void testGetMessageDigest() {
+        final DigestUtils digestUtils = new DigestUtils(MessageDigestAlgorithms.MD5);
+        assertNotNull(digestUtils.getMessageDigest());
+        assertEquals(MessageDigestAlgorithms.MD5, digestUtils.getMessageDigest().getAlgorithm());
     }
 
     @Test(expected=IllegalArgumentException.class)
@@ -81,10 +139,17 @@ public class DigestUtilsTest {
         DigestUtils.getDigest("Bogus Bogus");
     }
 
+    @Test
+    public void testIsAvailable() {
+        assertTrue(DigestUtils.isAvailable(MessageDigestAlgorithms.MD5));
+        assertFalse(DigestUtils.isAvailable("FOO"));
+        assertFalse(DigestUtils.isAvailable(null));
+    }
+
     @Test
     public void testMd2Hex() throws IOException {
         // Examples from RFC 1319
-        assertEquals("8350e5a3e24c153df2275c9f80692773", DigestUtils.md2Hex(""));
+        assertEquals("8350e5a3e24c153df2275c9f80692773", DigestUtils.md2Hex(EMPTY_STRING));
 
         assertEquals("32ec01ec4a6dac72c0ab96fb34c0b5d1", DigestUtils.md2Hex("a"));
 
@@ -137,7 +202,7 @@ public class DigestUtilsTest {
     @Test
     public void testMd5Hex() throws IOException {
         // Examples from RFC 1321
-        assertEquals("d41d8cd98f00b204e9800998ecf8427e", DigestUtils.md5Hex(""));
+        assertEquals("d41d8cd98f00b204e9800998ecf8427e", DigestUtils.md5Hex(EMPTY_STRING));
 
         assertEquals("0cc175b9c0f1b6a831c399e269772661", DigestUtils.md5Hex("a"));
 
@@ -157,7 +222,7 @@ public class DigestUtilsTest {
 
         assertEquals(DigestUtils.md5Hex(testData),
                 DigestUtils.md5Hex(new ByteArrayInputStream(testData)));
-}
+    }
 
     /**
      * An MD5 hash converted to hex should always be 32 characters.
@@ -256,10 +321,33 @@ public class DigestUtilsTest {
     }
 
     @Test
-    public void testSha224() throws IOException {
+    public void testSha224_FileAsHex() throws IOException {
+        assumeJava8();
+        final String expected = "d14a028c2a3a2bc9476102bb288234c415a2b01f828ea62ac5b3e42f";
+        final String pathname = "src/test/resources/empty.bin";
+        final String algo = MessageDigestAlgorithms.SHA_224;
+        final DigestUtils digestUtils = new DigestUtils(algo);
+        assertEquals(expected, digestUtils.digestAsHex(new File(pathname)));
+        try (final FileInputStream inputStream = new FileInputStream(pathname)) {
+            assertEquals(expected, digestUtils.digestAsHex(inputStream));
+        }
+        final byte[] allBytes = Files.readAllBytes(Paths.get(pathname));
+        assertEquals(expected, digestUtils.digestAsHex(allBytes));
+        assertEquals(expected, digestUtils.digestAsHex(ByteBuffer.wrap(allBytes)));
+    }
+
+    @Test
+    public void testSha224_PathAsHex() throws IOException {
         assumeJava8();
         assertEquals("d14a028c2a3a2bc9476102bb288234c415a2b01f828ea62ac5b3e42f",
-                new DigestUtils(MessageDigestAlgorithms.SHA_224).digestAsHex(("")));
+                new DigestUtils(MessageDigestAlgorithms.SHA_224).digestAsHex(Paths.get("src/test/resources/empty.bin")));
+    }
+
+    @Test
+    public void testSha224_StringAsHex() {
+        assumeJava8();
+        assertEquals("d14a028c2a3a2bc9476102bb288234c415a2b01f828ea62ac5b3e42f",
+                new DigestUtils(MessageDigestAlgorithms.SHA_224).digestAsHex(EMPTY_STRING));
         assertEquals("730e109bd7a8a32b1cb9d9a09aa2325d2430587ddbc0c38bad911525",
                 new DigestUtils(MessageDigestAlgorithms.SHA_224).digestAsHex("The quick brown fox jumps over the lazy dog"));
 
@@ -280,6 +368,84 @@ public class DigestUtilsTest {
             DigestUtils.sha256Hex(new ByteArrayInputStream(testData)));
     }
 
+    @Test
+    public void testSha256HexInputStream() throws IOException {
+        assertEquals(DigestUtils.sha256Hex(testData),
+                DigestUtils.sha256Hex(new ByteArrayInputStream(testData)));
+    }
+
+    @Test
+    public void testSha3_224() {
+        assumeJava9();
+        // Examples from https://csrc.nist.gov/projects/cryptographic-standards-and-guidelines/example-values
+        //
+        // https://csrc.nist.gov/CSRC/media/Projects/Cryptographic-Standards-and-Guidelines/documents/examples/SHA3-224_Msg0.pdf
+        assertEquals(
+                "6b4e03423667dbb73b6e15454f0eb1abd4597f9a1b078e3f5b5a6bc7",
+                DigestUtils.sha3_224Hex(EMPTY_STRING));
+    }
+
+    @Test
+    public void testSha3_224HexInputStream() throws IOException {
+        assumeJava9();
+        assertEquals(DigestUtils.sha3_224Hex(testData),
+                DigestUtils.sha3_224Hex(new ByteArrayInputStream(testData)));
+    }
+
+    @Test
+    public void testSha3_256() {
+        assumeJava9();
+        // Examples from https://csrc.nist.gov/projects/cryptographic-standards-and-guidelines/example-values
+        //
+        // https://csrc.nist.gov/CSRC/media/Projects/Cryptographic-Standards-and-Guidelines/documents/examples/SHA3-256_Msg0.pdf
+        assertEquals(
+                "a7ffc6f8bf1ed76651c14756a061d662f580ff4de43b49fa82d80a4b80f8434a",
+                DigestUtils.sha3_256Hex(EMPTY_STRING));
+    }
+
+    @Test
+    public void testSha3_256HexInputStream() throws IOException {
+        assumeJava9();
+        assertEquals(DigestUtils.sha3_256Hex(testData),
+                DigestUtils.sha3_256Hex(new ByteArrayInputStream(testData)));
+    }
+
+    @Test
+    public void testSha3_384() {
+        assumeJava9();
+        // Examples from https://csrc.nist.gov/projects/cryptographic-standards-and-guidelines/example-values
+        //
+        // https://csrc.nist.gov/CSRC/media/Projects/Cryptographic-Standards-and-Guidelines/documents/examples/SHA3-384_Msg0.pdf
+        assertEquals(
+                "0c63a75b845e4f7d01107d852e4c2485c51a50aaaa94fc61995e71bbee983a2ac3713831264adb47fb6bd1e058d5f004",
+                DigestUtils.sha3_384Hex(EMPTY_STRING));
+    }
+
+    @Test
+    public void testSha3_384HexInputStream() throws IOException {
+        assumeJava9();
+        assertEquals(DigestUtils.sha3_384Hex(testData),
+                DigestUtils.sha3_384Hex(new ByteArrayInputStream(testData)));
+    }
+
+    @Test
+    public void testSha3_512() {
+        assumeJava9();
+        // Examples from https://csrc.nist.gov/projects/cryptographic-standards-and-guidelines/example-values
+        //
+        // https://csrc.nist.gov/CSRC/media/Projects/Cryptographic-Standards-and-Guidelines/documents/examples/SHA3-512_Msg0.pdf
+        assertEquals(
+                "a69f73cca23a9ac5c8b567dc185a756e97c982164fe25859e0d1dcc1475c80a615b2123af1f5f94c11e3e9402c3ac558f500199d95b6d3e301758586281dcd26",
+                DigestUtils.sha3_512Hex(EMPTY_STRING));
+    }
+
+    @Test
+    public void testSha3_512HexInputStream() throws IOException {
+        assumeJava9();
+        assertEquals(DigestUtils.sha3_512Hex(testData),
+                DigestUtils.sha3_512Hex(new ByteArrayInputStream(testData)));
+    }
+
     @Test
     public void testSha384() throws IOException {
     // Examples from FIPS 180-2
@@ -298,7 +464,13 @@ public class DigestUtilsTest {
     }
 
     @Test
-    public void testSha512() throws IOException {
+    public void testSha384HexInputStream() throws IOException {
+        assertEquals(DigestUtils.sha384Hex(testData),
+                DigestUtils.sha384Hex(new ByteArrayInputStream(testData)));
+    }
+
+    @Test
+    public void testSha512() {
     // Examples from FIPS 180-2
     assertEquals("ddaf35a193617abacc417349ae20413112e6fa4e89a97ea20a9eeee64b55d39a" +
             "2192992a274fc1a836ba3c23a3feebbd454d4423643ce80e2a9ac94fa54ca49f",
@@ -312,6 +484,53 @@ public class DigestUtilsTest {
                        "hijklmnoijklmnopjklmnopqklmnopqrlmnopqrsmnopqrstnopqrstu"));
     }
 
+    @Test
+    public void testSha512_224() throws Exception {
+        assumeJava9();
+        // Examples from
+        // https://csrc.nist.gov/CSRC/media/Projects/Cryptographic-Standards-and-Guidelines/documents/examples/SHA512_224.pdf
+        final String stringInput = "abc";
+        final byte[] bytesInput = getBytesUtf8(stringInput);
+        final String resultString = "4634270F707B6A54DAAE7530460842E20E37ED265CEEE9A43E8924AA".toLowerCase(Locale.ROOT);
+        final byte[] resultBytes = Hex.decodeHex(resultString);
+        //
+        assertArrayEquals(resultBytes, DigestUtils.sha512_224(bytesInput));
+        assertArrayEquals(resultBytes, DigestUtils.sha512_224(new ByteArrayInputStream(bytesInput)));
+        assertArrayEquals(resultBytes, DigestUtils.sha512_224(stringInput));
+        //
+        assertEquals(resultString, DigestUtils.sha512_224Hex(bytesInput));
+        assertEquals(resultString, DigestUtils.sha512_224Hex(new ByteArrayInputStream(bytesInput)));
+        assertEquals(resultString, DigestUtils.sha512_224Hex(stringInput));
+        // Example 2
+        assertEquals("23FEC5BB94D60B23308192640B0C453335D664734FE40E7268674AF9".toLowerCase(Locale.ROOT),
+            DigestUtils.sha512_224Hex(
+                "abcdefghbcdefghicdefghijdefghijkefghijklfghijklmghijklmnhijklmnoijklmnopjklmnopqklmnopqrlmnopqrsmnopqrstnopqrstu"));
+    }
+
+    @Test
+    public void testSha512_256() throws Exception {
+        assumeJava9();
+        // Examples from
+        // https://csrc.nist.gov/CSRC/media/Projects/Cryptographic-Standards-and-Guidelines/documents/examples/SHA512_256.pdf
+        final String stringInput = "abc";
+        final byte[] bytesInput = getBytesUtf8(stringInput);
+        final String resultString = "53048E2681941EF99B2E29B76B4C7DABE4C2D0C634FC6D46E0E2F13107E7AF23"
+            .toLowerCase(Locale.ROOT);
+        final byte[] resultBytes = Hex.decodeHex(resultString);
+        //
+        assertArrayEquals(resultBytes, DigestUtils.sha512_256(bytesInput));
+        assertArrayEquals(resultBytes, DigestUtils.sha512_256(new ByteArrayInputStream(bytesInput)));
+        assertArrayEquals(resultBytes, DigestUtils.sha512_256(stringInput));
+        //
+        assertEquals(resultString, DigestUtils.sha512_256Hex(bytesInput));
+        assertEquals(resultString, DigestUtils.sha512_256Hex(new ByteArrayInputStream(bytesInput)));
+        assertEquals(resultString, DigestUtils.sha512_256Hex(stringInput));
+        // Example 2
+        assertEquals("3928E184FB8690F840DA3988121D31BE65CB9D3EF83EE6146FEAC861E19B563A".toLowerCase(Locale.ROOT),
+            DigestUtils.sha512_256Hex(
+                "abcdefghbcdefghicdefghijdefghijkefghijklfghijklmghijklmnhijklmnoijklmnopjklmnopqklmnopqrlmnopqrsmnopqrstnopqrstu"));
+    }
+
     @Test
     public void testSha512HexInputStream() throws IOException {
         assertEquals(DigestUtils.sha512Hex(testData),
